{
magnifier.dpr

Project file and main source file

Copyright (C) 1998 - 2007 Harri Pyy, Chris O'Donnell, Felipe Monteiro de Carvalho

This file is part of Virtual Magnifying Glass.

Virtual Magnifying Glass is free software;
you can redistribute it and/or modify it under the
terms of the GNU General Public License version 2
as published by the Free Software Foundation.

Virtual Magnifying Glass is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE. See the GNU General Public License for more details.

Please note that the General Public License version 2 does not permit
incorporating Virtual Magnifying Glass into proprietary
programs.

AUTHORS: Chris O'Donnell, Felipe Monteiro de Carvalho and Harri Pyy
}
program lente;

{*******************************************************************
*  Compatibility preprocessor code to allow the project to be compiled
* both on the Free Pascal Compiler and on Borland Delphi
*******************************************************************}
{$IFDEF FPC}
  {$MODE DELPHI}{$H+}
{$ENDIF}

{$IFDEF Win32}
  {$DEFINE Windows}
{$ENDIF}

{*******************************************************************
*  RC and RES Resource files are Windows only
*  Separate files for Delphi and FPC avoids the need to rebuild it
*******************************************************************}
{$IFDEF Windows}
  {$IFDEF FPC}
    {$R fpcmagnifier.rc}
  {$ELSE}
    {$R magnifier.res}
  {$ENDIF}
{$ENDIF}

uses
{$IFDEF FPC}
  Interfaces,
  LResources,
{$ENDIF}
{$IFDEF Windows}
  Windows,
{$ENDIF}
  Forms,
  glass in 'glass.pas',
  constants in 'constants.pas',
  appsettings in 'appsettings.pas',
  app in 'app.pas',
  about in 'about.pas',
  plugins in 'plugins.pas',
  translationsvmg in 'translationsvmg.pas',
  configdlg in 'configdlg.pas',
//  lazfmlviewer in 'lazfmlviewer.pas',
//  fmlscan in 'fmldriver.pas',
  plugininfo;

{*******************************************************************
*  Application Main Procedure
*
*  DESCRIPTION:    This procedure is executed as soon as the program is executed
*
*  PARAMETERS:     None
*
*  RETURNS:        Nothing
*
*******************************************************************}
var
  AHandle, Semaphore: Cardinal;
begin
  {*******************************************************************
  *  LRS Resource file for linux
  *******************************************************************}
{$IFDEF Unix}
  {$I magnifier.lrs}
{$ENDIF}

  {*******************************************************************
  *  Verifies if another instance is already running by using semaphores
  *******************************************************************}
  {$IFDEF Windows}
    Semaphore := OpenSemaphore(EVENT_ALL_ACCESS, False, szAppTitle);

    // Noone is holding the semaphore, so let's create one
    if Semaphore = 0 then
     Semaphore := CreateSemaphore(nil, 3, 3, szAppTitle)
    else
    begin
      AHandle := FindWindow(nil, szAppTitle);
      SendMessage(AHandle, MYWM_SHOWGLASS, 0, 0);
      Exit;
    end;
  {$ENDIF}

  {*******************************************************************
  *  Program initialization
  *
  *  The order in which these methods are called is very important
  *******************************************************************}
  Application.Initialize;
  Application.CreateForm(TMainWindow, vMainWindow);
  Application.CreateForm(TAboutWindow, vAboutWindow);
  Application.CreateForm(TConfigDialog, vConfigDialog);
//  Application.CreateForm(TFMLViewerDialog, vFMLViewerDialog);
  
  {*******************************************************************
  *  Features that need to be initialized at software startup are
  * processed here
  *******************************************************************}
  Application.ShowMainForm := vConfigurations.showWhenExecuted
   and (not vConfigurations.UsePlugins);

  { Initializes global shortcuts for the dynamic mode }
  {$IFDEF Windows}
    {$IFNDEF WinCE}
      RegisterHotKey(vMainWindow.Handle, 1, MOD_ALT or MOD_CONTROL, VK_LEFT);
      RegisterHotKey(vMainWindow.Handle, 2, MOD_ALT or MOD_CONTROL, VK_UP);
      RegisterHotKey(vMainWindow.Handle, 3, MOD_ALT or MOD_CONTROL, VK_RIGHT);
      RegisterHotKey(vMainWindow.Handle, 4, MOD_ALT or MOD_CONTROL, VK_DOWN);
    {$ENDIF}
  {$ENDIF}

  { Initializes HotKey }
  vMainWindow.UpdateHotKey(nil);
  {*******************************************************************
  * Enters the Message Loop
  *******************************************************************}
  Application.Run;

  {*******************************************************************
  *  Finalization - Release the semaphore for future instances
  *******************************************************************}
  {$IFDEF Windows}
    if Semaphore <> 0 then CloseHandle(Semaphore);
  {$ENDIF}
end.

